==> pgadmin3-1.0.0.tar.gz: pgadmin3-1.0.0/src/schema/pgTrigger.cpp <==
//////////////////////////////////////////////////////////////////////////
//
// pgAdmin III - PostgreSQL Tools
// Copyright (C) 2002 - 2003, The pgAdmin Development Team
// This software is released under the Artistic Licence
//
// pgTrigger.cpp - Trigger class
//
//////////////////////////////////////////////////////////////////////////

// wxWindows headers
#include <wx/wx.h>

// App headers
#include "pgAdmin3.h"
#include "misc.h"
#include "pgDefs.h"
#include "pgObject.h"

#include "pgTrigger.h"
#include "pgCollection.h"
#include "pgFunction.h"


pgTrigger::pgTrigger(pgSchema *newSchema, const wxString& newName)
: pgSchemaObject(newSchema, PG_TRIGGER, newName)
{
    triggerFunction=0;
}


pgTrigger::~pgTrigger()
{
    if (!expandedKids && triggerFunction)
    {
        // triggerFunction wasn't appended to tree, so we 
        // need to delete it manually
        delete triggerFunction;
    }
}

bool pgTrigger::DropObject(wxFrame *frame, wxTreeCtrl *browser)
{
    return GetDatabase()->ExecuteVoid(wxT("DROP TRIGGER ") + GetQuotedIdentifier() + wxT(" ON ") + GetQuotedFullTable());
}


void pgTrigger::SetDirty()
{
    if (expandedKids)
        triggerFunction=0;
    pgObject::SetDirty();
}


wxString pgTrigger::GetSql(wxTreeCtrl *browser)
{
    if (sql.IsNull() && this->triggerFunction)
    {
        sql = wxT("-- Trigger: ") + GetName() + wxT(" on ") + GetQuotedFullTable() + wxT("\n\n")
            + wxT("-- DROP TRIGGER ") + GetName()
            + wxT(" ON ") + GetQuotedFullTable() +wxT(";\n\nCREATE TRIGGER ") + GetName()
            + wxT("\n  ") + GetFireWhen() 
            + wxT(" ") + GetEvent()
            + wxT("\n  ON ") + GetQuotedFullTable()
            + wxT("\n  FOR EACH ") + GetForEach()
            + wxT("\n  EXECUTE PROCEDURE ") + triggerFunction->GetQuotedFullIdentifier() 
            + wxT("(") + GetArguments() + wxT(")")
            + wxT(";\n");

        if (!GetComment().IsEmpty())
            sql += wxT("COMMENT ON TRIGGER ") + GetQuotedIdentifier() + wxT(" ON ") + GetQuotedFullTable()
                +  wxT(" IS ") + qtString(GetComment()) + wxT(";\n");
    }

    return sql;
}


wxString pgTrigger::GetFireWhen() const 
{
    return (triggerType & TRIGGER_TYPE_BEFORE) ? wxT("BEFORE") : wxT("AFTER");
}


wxString pgTrigger::GetEvent() const
{
    wxString event;
    if (triggerType & TRIGGER_TYPE_INSERT)
        event = wxT("INSERT");
    if (triggerType & TRIGGER_TYPE_UPDATE)
    {
        if (!event.IsNull())
            event += wxT(" OR ");
        event += wxT("UPDATE");
    }
    if (triggerType & TRIGGER_TYPE_DELETE)
    {
        if (!event.IsNull())
            event += wxT(" OR ");
        event += wxT("DELETE");
    }
    return event;
}

wxString pgTrigger::GetForEach() const
{
    return (triggerType & TRIGGER_TYPE_ROW) ? wxT("ROW") : wxT("STATEMENT");
}



void pgTrigger::ShowTreeDetail(wxTreeCtrl *browser, frmMain *form, wxListCtrl *properties, wxListCtrl *statistics, ctlSQLBox *sqlPane)
{
    if (!expandedKids)
    {
        if (browser)
        {
            // if no browser present, function will not be appended to tree
            expandedKids = true;
        }
        if (triggerFunction)
            delete triggerFunction;

        // append function here
        triggerFunction=pgFunction::AppendFunctions(this, GetSchema(), browser, wxT(
            "WHERE pr.oid=") + NumToStr(functionOid) + wxT("::oid\n"));
        if (triggerFunction)
        {
            iSetFunction(triggerFunction->GetName());
        }
    }

    if (properties)
    {
        CreateListColumns(properties);
        int pos=0;

        InsertListItem(properties, pos++, _("Name"), GetName());
        InsertListItem(properties, pos++, _("OID"), GetOid());
        InsertListItem(properties, pos++, _("Fires"), GetFireWhen());
        InsertListItem(properties, pos++, _("Event"), GetEvent());
        InsertListItem(properties, pos++, _("For each"), GetForEach());
        InsertListItem(properties, pos++, _("Function"), GetFunction() + wxT("(") + GetArguments() + wxT(")"));
        InsertListItem(properties, pos++, _("Enabled?"), GetEnabled());
        InsertListItem(properties, pos++, _("System trigger?"), GetSystemObject());
        InsertListItem(properties, pos++, _("Comment"), GetComment());
    }
}



pgObject *pgTrigger::Refresh(wxTreeCtrl *browser, const wxTreeItemId item)
{
    pgObject *trigger=0;
    wxTreeItemId parentItem=browser->GetItemParent(item);
    if (parentItem)
    {
        pgObject *obj=(pgObject*)browser->GetItemData(parentItem);
        if (obj->GetType() == PG_TRIGGERS)
            trigger = ReadObjects((pgCollection*)obj, 0, wxT("\n   AND t.oid=") + GetOidStr());
    }
    return trigger;
}



pgObject *pgTrigger::ReadObjects(pgCollection *collection, wxTreeCtrl *browser, const wxString &restriction)
{
    pgTrigger *trigger=0;

        pgSet *triggers= collection->GetDatabase()->ExecuteSet(
        wxT("SELECT t.oid, t.*, relname, nspname, des.description\n")
        wxT("  FROM pg_trigger t\n")
        wxT("  JOIN pg_class cl ON cl.oid=tgrelid\n")
        wxT("  JOIN pg_namespace na ON na.oid=relnamespace\n")
        wxT("  LEFT OUTER JOIN pg_description des ON des.objoid=t.oid\n")
        wxT(" WHERE NOT tgisconstraint AND tgrelid = ") + collection->GetOidStr() + wxT("\n")
        wxT(" ORDER BY tgname"));

    if (triggers)
    {
        while (!triggers->Eof())
        {
            trigger = new pgTrigger(collection->GetSchema(), triggers->GetVal(wxT("tgname")));

            trigger->iSetOid(triggers->GetOid(wxT("oid")));
            trigger->iSetTableOid(collection->GetOid());
            trigger->iSetComment(triggers->GetVal(wxT("description")));
            trigger->iSetFunctionOid(triggers->GetOid(wxT("tgfoid")));
            trigger->iSetEnabled(triggers->GetBool(wxT("tgenabled")));
            trigger->iSetTriggerType(triggers->GetLong(wxT("tgtype")));
            trigger->iSetQuotedFullTable(qtIdent(triggers->GetVal(wxT("nspname")))+wxT(".")+qtIdent(triggers->GetVal(wxT("relname"))));
            wxString arglist=triggers->GetVal(wxT("tgargs"));
            wxString args;

            while (!arglist.IsEmpty())
            {
                int pos=arglist.Find(wxT("\\000"));
                if (pos != 0)
                {
                    wxString arg;
                    if (pos > 0)
                        arg=arglist.Left(pos);
                    else
                        arg=arglist;

                    if (!args.IsEmpty())
                        args += wxT(", ");
                    if (NumToStr(StrToLong(arg)) == arg)
                        args += arg;
                    else
                        args += qtString(arg);
                }
                if (pos >= 0)
                    arglist = arglist.Mid(pos+4);
                else
                    break;
            }
            trigger->iSetArguments(args);

            if (browser)
            {
                collection->AppendBrowserItem(browser, trigger);
    			triggers->MoveNext();
            }
            else
                break;
        }

		delete triggers;
    }
    return trigger;
}

==> pgadmin3_1.0.2.orig.tar.gz: pgadmin3-1.0.2/src/schema/pgTrigger.cpp <==
//////////////////////////////////////////////////////////////////////////
//
// pgAdmin III - PostgreSQL Tools
// Copyright (C) 2002 - 2003, The pgAdmin Development Team
// This software is released under the Artistic Licence
//
// pgTrigger.cpp - Trigger class
//
//////////////////////////////////////////////////////////////////////////

// wxWindows headers
#include <wx/wx.h>

// App headers
#include "pgAdmin3.h"
#include "misc.h"
#include "pgDefs.h"
#include "pgObject.h"

#include "pgTrigger.h"
#include "pgCollection.h"
#include "pgFunction.h"


pgTrigger::pgTrigger(pgSchema *newSchema, const wxString& newName)
: pgSchemaObject(newSchema, PG_TRIGGER, newName)
{
    triggerFunction=0;
}


pgTrigger::~pgTrigger()
{
    if (!expandedKids && triggerFunction)
    {
        // triggerFunction wasn't appended to tree, so we 
        // need to delete it manually
        delete triggerFunction;
    }
}

bool pgTrigger::DropObject(wxFrame *frame, wxTreeCtrl *browser)
{
    return GetDatabase()->ExecuteVoid(wxT("DROP TRIGGER ") + GetQuotedIdentifier() + wxT(" ON ") + GetQuotedFullTable());
}


void pgTrigger::SetDirty()
{
    if (expandedKids)
        triggerFunction=0;
    pgObject::SetDirty();
}


wxString pgTrigger::GetSql(wxTreeCtrl *browser)
{
    if (sql.IsNull() && this->triggerFunction)
    {
        sql = wxT("-- Trigger: ") + GetName() + wxT(" on ") + GetQuotedFullTable() + wxT("\n\n")
            + wxT("-- DROP TRIGGER ") + GetName()
            + wxT(" ON ") + GetQuotedFullTable() +wxT(";\n\nCREATE TRIGGER ") + GetName()
            + wxT("\n  ") + GetFireWhen() 
            + wxT(" ") + GetEvent()
            + wxT("\n  ON ") + GetQuotedFullTable()
            + wxT("\n  FOR EACH ") + GetForEach()
            + wxT("\n  EXECUTE PROCEDURE ") + triggerFunction->GetQuotedFullIdentifier() 
            + wxT("(") + GetArguments() + wxT(")")
            + wxT(";\n");

        if (!GetComment().IsEmpty())
            sql += wxT("COMMENT ON TRIGGER ") + GetQuotedIdentifier() + wxT(" ON ") + GetQuotedFullTable()
                +  wxT(" IS ") + qtString(GetComment()) + wxT(";\n");
    }

    return sql;
}


wxString pgTrigger::GetFireWhen() const 
{
    return (triggerType & TRIGGER_TYPE_BEFORE) ? wxT("BEFORE") : wxT("AFTER");
}


wxString pgTrigger::GetEvent() const
{
    wxString event;
    if (triggerType & TRIGGER_TYPE_INSERT)
        event = wxT("INSERT");
    if (triggerType & TRIGGER_TYPE_UPDATE)
    {
        if (!event.IsNull())
            event += wxT(" OR ");
        event += wxT("UPDATE");
    }
    if (triggerType & TRIGGER_TYPE_DELETE)
    {
        if (!event.IsNull())
            event += wxT(" OR ");
        event += wxT("DELETE");
    }
    return event;
}

wxString pgTrigger::GetForEach() const
{
    return (triggerType & TRIGGER_TYPE_ROW) ? wxT("ROW") : wxT("STATEMENT");
}



void pgTrigger::ShowTreeDetail(wxTreeCtrl *browser, frmMain *form, wxListCtrl *properties, wxListCtrl *statistics, ctlSQLBox *sqlPane)
{
    if (!expandedKids)
    {
        if (browser)
        {
            // if no browser present, function will not be appended to tree
            expandedKids = true;
        }
        if (triggerFunction)
            delete triggerFunction;

        // append function here
        triggerFunction=pgFunction::AppendFunctions(this, GetSchema(), browser, wxT(
            "WHERE pr.oid=") + NumToStr(functionOid) + wxT("::oid\n"));
        if (triggerFunction)
        {
            iSetFunction(triggerFunction->GetName());
        }
    }

    if (properties)
    {
        CreateListColumns(properties);
        int pos=0;

        InsertListItem(properties, pos++, _("Name"), GetName());
        InsertListItem(properties, pos++, _("OID"), GetOid());
        InsertListItem(properties, pos++, _("Fires"), GetFireWhen());
        InsertListItem(properties, pos++, _("Event"), GetEvent());
        InsertListItem(properties, pos++, _("For each"), GetForEach());
        InsertListItem(properties, pos++, _("Function"), GetFunction() + wxT("(") + GetArguments() + wxT(")"));
        InsertListItem(properties, pos++, _("Enabled?"), GetEnabled());
        InsertListItem(properties, pos++, _("System trigger?"), GetSystemObject());
        InsertListItem(properties, pos++, _("Comment"), GetComment());
    }
}



pgObject *pgTrigger::Refresh(wxTreeCtrl *browser, const wxTreeItemId item)
{
    pgObject *trigger=0;
    wxTreeItemId parentItem=browser->GetItemParent(item);
    if (parentItem)
    {
        pgObject *obj=(pgObject*)browser->GetItemData(parentItem);
        if (obj->GetType() == PG_TRIGGERS)
            trigger = ReadObjects((pgCollection*)obj, 0, wxT("\n   AND t.oid=") + GetOidStr());
    }
    return trigger;
}



pgObject *pgTrigger::ReadObjects(pgCollection *collection, wxTreeCtrl *browser, const wxString &restriction)
{
    pgTrigger *trigger=0;

    wxString trig_sql;
    trig_sql = wxT("SELECT t.oid, t.*, relname, nspname, des.description\n")
        wxT("  FROM pg_trigger t\n")
        wxT("  JOIN pg_class cl ON cl.oid=tgrelid\n")
        wxT("  JOIN pg_namespace na ON na.oid=relnamespace\n")
        wxT("  LEFT OUTER JOIN pg_description des ON des.objoid=t.oid\n")
               wxT(" WHERE NOT tgisconstraint \n");
    if (restriction.IsEmpty())
        trig_sql += wxT(" AND tgrelid = ") + collection->GetOidStr() + wxT("\n");
    else
        trig_sql += restriction + wxT("\n");
    trig_sql += wxT(" ORDER BY tgname");
    pgSet *triggers= collection->GetDatabase()->ExecuteSet(trig_sql);

    if (triggers)
    {
        while (!triggers->Eof())
        {
            trigger = new pgTrigger(collection->GetSchema(), triggers->GetVal(wxT("tgname")));

            trigger->iSetOid(triggers->GetOid(wxT("oid")));
            trigger->iSetTableOid(collection->GetOid());
            trigger->iSetComment(triggers->GetVal(wxT("description")));
            trigger->iSetFunctionOid(triggers->GetOid(wxT("tgfoid")));
            trigger->iSetEnabled(triggers->GetBool(wxT("tgenabled")));
            trigger->iSetTriggerType(triggers->GetLong(wxT("tgtype")));
            trigger->iSetQuotedFullTable(qtIdent(triggers->GetVal(wxT("nspname")))+wxT(".")+qtIdent(triggers->GetVal(wxT("relname"))));
            wxString arglist=triggers->GetVal(wxT("tgargs"));
            wxString args;

            while (!arglist.IsEmpty())
            {
                int pos=arglist.Find(wxT("\\000"));
                if (pos != 0)
                {
                    wxString arg;
                    if (pos > 0)
                        arg=arglist.Left(pos);
                    else
                        arg=arglist;

                    if (!args.IsEmpty())
                        args += wxT(", ");
                    if (NumToStr(StrToLong(arg)) == arg)
                        args += arg;
                    else
                        args += qtString(arg);
                }
                if (pos >= 0)
                    arglist = arglist.Mid(pos+4);
                else
                    break;
            }
            trigger->iSetArguments(args);

            if (browser)
            {
                collection->AppendBrowserItem(browser, trigger);
    			triggers->MoveNext();
            }
            else
                break;
        }

		delete triggers;
    }
    return trigger;
}
